@@ -9,6 +9,9 @@ use Carp;
 use Config;
 use POSIX qw{ uname };
 
+# This occurs in both inc/My/Module/Meta.pm and inc/My/Module/Test.pm
+use constant CAN_USE_UNICODE	=> "$]" >= 5.008004;
+
 sub new {
     my ( $class ) = @_;
     ref $class and $class = ref $class;
@@ -24,7 +27,13 @@ sub abstract {
 }
 
 sub add_to_cleanup {
-    return [ qw{ cover_db *.gcov *.gcda *.gcno xt/author/optionals } ];
+    return [
+	qw{ pbl cover_db *.gcov *.gcda *.gcno xt/author/optionals },
+	map { "lib/Mac/$_" }
+	    qw{ Pasteboard.xs constant-c.inc constant-h.inc
+	    constant-xs.inc pbl.c pbl.h ppport.h },
+
+    ];
 }
 
 sub author {
@@ -35,25 +44,48 @@ sub build_requires {
     my ( undef, @extra ) = @_;		# Invocant not used
     return +{
 	'ExtUtils::CBuilder'	=> 0,
+	( CAN_USE_UNICODE ?
+	    ( 'I18N::Langinfo'	=> 0 ) :
+	    () ),
 	'Test::More'	=> 0.96,	# Because of subtest().
 	@extra,
     };
 }
 
 sub ccflags {
+    my ( undef, $opt ) = @_;
+
+    $opt
+	or confess( 'Bug - \%opt argument required' );
+
     my @ccflags;
 
+    CAN_USE_UNICODE
+	and push @ccflags, '-DPERL_CAN_USE_UNICODE';
+
     my ( $darwin_version ) = split qr{ [.] }smx, ( uname() )[2];
     $darwin_version >= 8
 	and push @ccflags, '-DTIGER';
 
-    $darwin_version >= 19
-	and push @ccflags, '-DCATALINA';
+    # Darwin 15 = macOS 10.11 = El Capitan
+    if ( $opt->{t} ) {
+	$opt->{u}
+	    and die "Please do not assert both -t and -u. It is too confusing.\n";
+    } elsif ( $opt->{u} || $darwin_version >= 15 ) {
+	push @ccflags, '-DUTF_8_PLAIN_TEXT';
+    }
+
+    $opt->{p}
+	and push @ccflags, qw{ -DUSE_PBL_BACKEND };
 
     system "$Config{cc} -fsyntax-only inc/trytypes.c 2>/dev/null";
     $?
 	or push @ccflags, '-DUSE_MACTYPES';
 
+    system "$Config{cc} -Werror -Wno-compound-token-split-by-macro -o /dev/null inc/true.c 2>/dev/null";
+    $?
+	or push @ccflags, '-Wno-compound-token-split-by-macro';
+
     if ( my $debug = $ENV{DEVELOPER_DEBUG} ) {
 	push @ccflags, '-DDEBUG_PBL';
 	$debug =~ m/ \b backtrace \b /smxi
@@ -65,8 +97,8 @@ sub ccflags {
 sub configure_requires {
     return +{
 	'Config'		=> 0,
-	'File::Basename'	=> 0,
 	'Getopt::Std'		=> 0,
+	'Scalar::Util'		=> 0,
 	'lib'		=> 0,
 	'strict'	=> 0,
 	'warnings'	=> 0,
@@ -96,7 +128,8 @@ sub meta_merge {
 	dynamic_config	=> 1,
 	resources	=> {
 	    bugtracker	=> {
-		web	=> 'https://github.com/trwyant/perl-Mac-Pasteboard/issues',
+		web	=> 'https://rt.cpan.org/Public/Dist/Display.html?Name=Mac-Pasteboard',
+		# web	=> 'https://github.com/trwyant/perl-Mac-Pasteboard/issues',
 		mailto  => 'wyant@cpan.org',
 	    },
 	    license	=> 'http://dev.perl.org/licenses/',
@@ -117,22 +150,56 @@ sub module_name {
 
 sub no_index {
     return +{
-      directory => [
-                     'inc',
-                     't',
-                     'xt',
-                   ],
+	directory => [ qw{ eg inc t xt } ],
     };
 }
 
 sub provides {
-    -d 'lib'
-	or return;
+    my $provides;
     local $@ = undef;
-    my $provides = eval {
+
+    eval {
+	require CPAN::Meta;
+	require ExtUtils::Manifest;
 	require Module::Metadata;
-	Module::Metadata->provides( version => 2, dir => 'lib' );
+
+	my $manifest;
+	{
+	    local $SIG{__WARN__} = sub {};
+	    $manifest = ExtUtils::Manifest::maniread();
+	}
+	keys %{ $manifest || {} }
+	    or return;
+
+	# Skeleton so we can use should_index_file() and
+	# should_index_package().
+	my $meta = CPAN::Meta->new( {
+		name	=> 'Euler',
+		version	=> 2.71828,
+		no_index	=> no_index(),
+	    },
+	);
+
+	# The Module::Metadata docs say not to use
+	# package_versions_from_directory() directly, but the 'files =>'
+	# version of provides() is broken, and has been known to be so
+	# since 2014, so it's not getting fixed any time soon. So:
+
+	foreach my $fn ( sort keys %{ $manifest } ) {
+	    $fn =~ m/ [.] pm \z /smx
+		or next;
+	    my $pvd = Module::Metadata->package_versions_from_directory(
+		undef, [ $fn ] );
+	    foreach my $pkg ( keys %{ $pvd } ) {
+		$meta->should_index_package( $pkg )
+		    and $meta->should_index_file( $pvd->{$pkg}{file} )
+		    and $provides->{$pkg} = $pvd->{$pkg};
+	    }
+	}
+
+	1;
     } or return;
+
     return ( provides => $provides );
 }
 
@@ -401,7 +468,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2010-2021 by Thomas R. Wyant, III
+Copyright (C) 2010-2022 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
